from tritise import Tritise
import fileinput
import click
from dateutil.parser import parse as parse_date

@click.group()
def cli():
    pass

@cli.command()
@click.argument('input_files', required = False, nargs = -1, default = None)
@click.option('--database', '-d', default = 'tritise.sqlite')
@click.option('--tag', '-t', default = None)
def read(input_files, database = None, tag = None):
    """Reads data from stdin or a file into the database"""
    t = Tritise(filename=database)
    print('Reading data...')
    for line in fileinput.input(input_files):
        print('Adding line: %s' % line)
        t.add(line.split()[0], tag=tag)
    print('done')

@cli.command()
@click.option('--from', '-f', 'start_string', default = None)
@click.option('--to', '-t', 'end_string', default = None)
@click.option('--database', '-d', default = 'tritise.sqlite')
@click.option('--tag', '-t', default = None)
def dump(start_string, end_string, database = None, tag = None):
    """Dumps a timeseries from a tritise database"""
    start_stamp = parse_date(start_string) if start_string else None
    end_stamp = parse_date(end_string) if end_string else None
    t = Tritise(filename=database)
    print('Dumping tag: %s' % (tag if tag else 'default'))
    entries = t.range(start_stamp, end_stamp, tag)
    for entry in entries:
        print(entry)

@cli.command()
@click.argument('value')
@click.option('--database', '-d', default = 'tritise.sqlite')
@click.option('--tag', '-t', default = None)
def add(value, database = None, tag = None):
    """Add a value with the current timestamp"""
    t = Tritise(filename=database)
    print('Adding value %s with tag: %s' % (value ,tag if tag else 'default'))
    print(t.add(value, tag))

if __name__ == '__main__':
    cli()


from sqlalchemy import create_engine, Column, Integer, String, DateTime, Float
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.sql import func
from sqlalchemy.orm import sessionmaker

import datetime

DEFAULT_DB_NAME = 'tritise.sqlite'

class NotConnectedError(Exception):
    pass

def require_cursor(f):
    def wrapper(*args):
        s = args[0]
        if not s.cursor:
            raise NotConnectedError()
        return f(*args)
    return wrapper

Base = declarative_base()

class TritiseEntry(Base):
    __tablename__ = 'data'

    id = Column(Integer, primary_key = True)
    timestamp = Column(DateTime, server_default=func.now())
    value = Column(Float, nullable=False)
    tag = Column(String)

    def __repr__(self):
        return '{}: {:>8.2f} {}'.format(self.timestamp, self.value, '[%s]' % self.tag if self.tag else '')

class Tritise(object):
    def __init__(self, filename = DEFAULT_DB_NAME, engine = None):
        """Without an engine and a filename a sqlite connection is assumed"""
        self._filename = filename
        self._engine = engine
        if self._engine is None:
            self._engine = create_engine('sqlite:///%s' % filename)
        Base.metadata.create_all(self._engine)
        self._sessionmaker = sessionmaker(bind=self._engine)
        self._session = self._sessionmaker()

    def add(self, value, tag = None, timestamp = None):
        """
        Add a data point with an optional tag to the time series.
        The entry is returned on success, None otherwise
        """
        t = TritiseEntry(value = value, tag = tag)
        if (timestamp):
            t.timestamp = timestamp
        self._session.add(t)
        self._session.commit()
        return t if t.id else None

    def add_many(self, data, tag = None):
        """
        Allows to add many existing data points with an optional tag.
        data must be an array of dicts with `value` and `timestamp` fields
        or behave like one
        """
        for point in data:
            point_tag = tag
            if 'tag' in point:
                point_tag = point['tag']
            t = TritiseEntry(value=point['value'], tag=point_tag, timestamp=point['timestamp'])
            self._session.add(t)
        return self._session.commit()

    def all(self, tag = None):
        return self._session.query(TritiseEntry)\
                   .order_by(TritiseEntry.timestamp.desc(), TritiseEntry.id.desc())\
                   .filter(TritiseEntry.tag == tag)\
                   .all()

    def last(self, tag = None):
        """Retrieve the newest data point for a tag"""
        return self._session.query(TritiseEntry)\
            .filter(TritiseEntry.tag == tag)\
            .order_by(TritiseEntry.timestamp.desc(), TritiseEntry.id.desc())\
            .first()

    def count(self, tag = None):
        """Count all entries with a common tag"""
        return self._session.query(func.count(TritiseEntry.id)).filter(TritiseEntry.tag == tag).scalar()

    def range(self, start_date = None, end_date = None, tag = None):
        """
        Retrieves all entries for a tag in a time range
        """
        query = self._session.query(TritiseEntry)\
                    .order_by(TritiseEntry.timestamp.desc(), TritiseEntry.id.desc())\
                    .filter(TritiseEntry.tag == tag)
        if start_date:
            query = query.filter(TritiseEntry.timestamp >= start_date)
        if end_date:
            query = query.filter(TritiseEntry.timestamp <= end_date)
        return query.all()


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tritise']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.0,<8.0',
 'python-dateutil>=2.8.1,<3.0.0',
 'sqlalchemy>=1.3.12,<2.0.0']

entry_points = \
{'console_scripts': ['tritise = tritise.cli:cli']}

setup_kwargs = {
    'name': 'tritise',
    'version': '0.1.0',
    'description': 'A small python library to handle trivial time series in an SQLite database.',
    'long_description': None,
    'author': 'asmw',
    'author_email': 'asmw@asmw.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
